import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Params } from '@angular/router';
import { Subject, from, switchMap, takeUntil, tap } from 'rxjs';
import { ComponentHostDirective } from '../directives/component-host.directive';
import { ServiceComponentType, servicesComponentFactory } from '../services-factory';
import { Service, ServiceTypes } from '../components';

@Component({
  selector: 'app-services',
  standalone: true,
  imports: [ComponentHostDirective],
  templateUrl: './services.component.html',
  styleUrls: ['./services.component.scss'],
})
export class ServicesComponent implements OnInit {
  @ViewChild(ComponentHostDirective, { static: true }) componentHost!: ComponentHostDirective;

  constructor(private readonly route: ActivatedRoute) { }

  destroy: Subject<void> = new Subject<void>();

  ngOnInit(): void { // subscribing to the stream when component has been created
    this.route.params
      .pipe(
        // switchMap(({ id }: Params) => from([]))),
        // tap((service: Service) => this.loadComponent(service)),
        takeUntil(this.destroy.asObservable()),
      )
      .subscribe(result => {
        this.loadComponent({
          id: result['id'],
          title: "string",
          description: "string"
        })
      });

  }

  loadComponent(service: Service): void {
    const { viewContainerRef }: ComponentHostDirective = this.componentHost;
    viewContainerRef.clear();

    const { instance } = viewContainerRef.createComponent<ServiceComponentType>(
      servicesComponentFactory[service.id as ServiceTypes]
    );
    // instance.data = service;
  }

  ngOnDestroy(): void {
    this.destroy.next();
  }
}
